#include "followupslistmodel.h"
#include <QFont>
#include <QColor>
#include <QRgb>
#include <QDateTime>
#include <QDate>

FollowupsListModel::FollowupsListModel(int ticketId, const QString & loginName, QObject *parent) :
    QAbstractTableModel(parent)
{
    this->parentTicketId = ticketId;
    this->loginName = loginName;
}

FollowupsListModel::FollowupsListModel(QObject *parent) :
    QAbstractTableModel(parent)
{
}

int FollowupsListModel::ticketId()
{
    return this->parentTicketId;
}

int FollowupsListModel::rowCount(const QModelIndex &parent) const
{
    if(parent.isValid())
        return 0;
    return this->rows.count();
}

int FollowupsListModel::columnCount(const QModelIndex &parent) const
{
    if(parent.isValid())
        return 0;
    if(this->rows.count() > 0)
    {
        QMultiMap<QString , QVariant> row = this->rows.at(0).toMap();
        return row.count();
    }
    else
        return 5;
}

QVariant FollowupsListModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
            return QVariant();

    if(role == Qt::TextAlignmentRole)
    {
        return QVariant(Qt::AlignLeft | Qt::AlignTop);
    }

    if (role == Qt::FontRole && index.column() == 3)
    {
        QVariant value;

        if( data(this->index(index.row(), 2), Qt::UserRole).toInt() < 0)
        {
            QFont font;
            font.setBold(true);
            value = qVariantFromValue(font);
        }

        return value;
    }

    if(role == Qt::BackgroundRole)
    {
        QVariant value;

        if( data(this->index(index.row(), 2), Qt::UserRole).toInt() < 0)
        {
            QColor color;
            color.setRgba(QRgb(0x80D6E3EF));
            value = color;
        }

        return value;
    }

    if (role == Qt::DisplayRole || role == Qt::UserRole)
    {
        QMultiMap<QString , QVariant> row = this->rows.at(index.row()).toMap();
        QVariant value;

        switch(index.column())
        {
            //ID
        case 0 : {
                value = row.value("followup_id");
                break;
            }
            //Дата
        case 1 : {
                QVariant dateTime = row.value("followup_date");
                value = dateTime.toDateTime().toString("dd.MM.yyyy hh:mm:ss");
                if(QDate::currentDate() == dateTime.toDate())
                        value = tr("today");
                if(QDate::currentDate().addDays(-1) == dateTime.toDate())
                        value = tr("yesterday");
                break;
            }
            //Владелец
        case 2 : {
                value = row.value("followup_user_id");
                break;
            }
            //Владелец (имя)
        case 3 : {
                if( data(this->index(index.row(), 2), Qt::UserRole).toInt() < 0)
                    value = this->loginName;
                else
                    value = row.value("followup_user_login");
                break;
            }
            //Содержание
        case 4 : {
                value = row.value("followup_content");
                break;
            }
        }

        return value;
    }

    return QVariant();
}

void FollowupsListModel::setupModel(const QList<QVariant> & rows)
{
    this->rows = rows;
    emit layoutChanged();
}

QVariant FollowupsListModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole)
    {
        if(orientation == Qt::Horizontal)
        {
            switch(section)
            {
            case 0 : {
                    return "№";
                    break;
                }
            case 1 : {
                    return tr("Date");
                    break;
                }
            case 2 : {
                    return tr("Autors id");
                    break;
                }
            case 3 : {
                    return tr("Autor");
                    break;
                }
            case 4 : {
                    return tr("Content");
                    break;
                }
            }
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

void FollowupsListModel::addFollowup(const QVariant & date, const QString & text)
{
    beginInsertRows(QModelIndex(), this->rowCount(),this->rowCount());

    QMultiMap<QString , QVariant> map;
    if(this->rowCount() > 0)
    {
        map = this->rows.at(this->rowCount()-1).toMap();
    }

    map.replace("followup_id", -1);
    map.replace("followup_date", date);
    map.replace("followup_user_id", -1);
    map.replace("followup_user_login", this->loginName);
    map.replace("followup_content", text);

    this->rows.append(map);

    endInsertRows();
}


