#include "login_dialog.h"
#include "ui_login_dialog.h"
#include <QMessageBox>
#include <QShowEvent>

LoginDialog::LoginDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::LoginDialog)
{
    ui->setupUi(this);

    ui->radioButtonAnonymous->setChecked(true);
    ui->radioButtonRegistered->setChecked(false);

    ui->stackedWidget->setCurrentIndex(0);

    on_radioButtonAnonymous_clicked();
}

LoginDialog::~LoginDialog()
{
    delete ui;
}

void LoginDialog::on_toolButtonLogin_clicked()
{
    if(ui->radioButtonAnonymous->isChecked())
    {
        if(ui->lineEditEmail->text().trimmed().length() == 0)
        {
            QMessageBox::warning(0,
                                     tr("Insufficient information!"),
                                     tr("Please, enter your email address!"),
                                     QMessageBox::Ok,Qt::NoButton);
            return;
        }
        emit this->loginAnonymous(ui->lineEditEmail->text());
        this->accept();
    }

    if(ui->radioButtonRegistered->isChecked())
    {
        emit this->loginRegistered(ui->lineEditLogin->text(), ui->lineEditPassword->text());
        this->accept();
    }
}

void LoginDialog::setType(int type)
{
    if(type == 0)
    {
        this->on_radioButtonAnonymous_clicked();
    }

    if(type == 1)
    {
        this->on_radioButtonRegistered_clicked();
    }
}

void LoginDialog::setLoginName(const QString & login)
{
    ui->lineEditLogin->setText(login);
    ui->lineEditPassword->setFocus();
}

void LoginDialog::on_radioButtonAnonymous_clicked()
{
    ui->radioButtonAnonymous->setChecked(true);
    ui->radioButtonRegistered->setChecked(false);
    ui->stackedWidget->setCurrentIndex(0);
    this->setWindowTitle(tr("Anonymous user"));
    ui->lineEditEmail->setFocus();
}

void LoginDialog::on_radioButtonRegistered_clicked()
{
    ui->radioButtonRegistered->setChecked(true);
    ui->radioButtonAnonymous->setChecked(false);
    ui->stackedWidget->setCurrentIndex(1);
    this->setWindowTitle(tr("Registered user"));
    ui->lineEditLogin->setFocus();
}

void LoginDialog::on_lineEditEmail_returnPressed()
{
    ui->toolButtonLogin->click();
}

void LoginDialog::on_lineEditLogin_returnPressed()
{
    ui->lineEditPassword->setFocus();
}

void LoginDialog::on_lineEditPassword_returnPressed()
{
    ui->toolButtonLogin->click();
}
