#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "login_dialog.h"
#include "mso_inifile.h"
#include <QMessageBox>
#include <QCloseEvent>
#include <QMenu>
#include "requestslistmodel.h"
#include "newticket_dialog.h"
#include <QDesktopWidget>
#include "chat_dialog.h"
#include "followupslistmodel.h"
#include <QTimer>
#include <QPainter>
#include "plasmanotify.h"
#include "requestlistdelegate.h"

using namespace xmlrpc;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    //устанавливаем кодеки
    QTextCodec::setCodecForCStrings( QTextCodec::codecForName("UTF-8") );
    QTextCodec::setCodecForTr( QTextCodec::codecForName("UTF-8") );
    QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));

    ui->setupUi(this);
    this->setupMainToolBar();
    RequestListDelegate * requestListDelegate = new RequestListDelegate(this);
    ui->tableView->setItemDelegate(requestListDelegate);

    //this->settingsObj = new MSO_IniFile(QCoreApplication::applicationDirPath()+"/sphere-client.ini");
    this->settingsObj = new MSO_IniFile("/etc/sphere-client.conf");
    this->sessionKey = "";

    this->model = new RequestsListModel();

    connect(this->model, SIGNAL(layoutChanged()), this, SLOT(doRefreshTicketBody()));

    ui->tableView->setModel(this->model);
    ui->tableView->hideColumn(4);
    ui->tableView->hideColumn(5);
    ui->tableView->hideColumn(6);
    ui->tableView->hideColumn(7);
    ui->tableView->hideColumn(8);
    ui->tableView->hideColumn(9);
    ui->tableView->hideColumn(10);

    ui->tableView->horizontalHeader()->setResizeMode(ui->tableView->horizontalHeader()->logicalIndexAt(0), QHeaderView::ResizeToContents);
    ui->tableView->horizontalHeader()->setResizeMode(1, QHeaderView::Stretch);
    ui->tableView->horizontalHeader()->setResizeMode(2, QHeaderView::ResizeToContents);
    ui->tableView->horizontalHeader()->setResizeMode(3, QHeaderView::ResizeToContents);
    ui->tableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);

    afterStarted = false;
    connected = false;

    counterNew = 0;
    counterAssign = 0;
    counterPlan = 0;
    counterWaiting = 0;
    counterSolved = 0;
    counterClosed = 0;
    counterFollowUps = 0;

    this->createActions();
    this->createTrayIcon();
    trayIcon->show();
    this->moveWindowToCenter();

    this->timer = new QTimer();

    QTimer::singleShot(2000, this, SLOT(applicationStarted()));

    this->wImageLabel = new QImage (":/images/icons/helpdesk_64.png");

    PlasmaNotify *notify = new PlasmaNotify("sphere-client", tr("Helpdesk agent Sphere"), tr("has been started"), ":/images/icons/helpdesk_64.png", 10000);
    notify->send();
}

MainWindow::~MainWindow()
{
    if(this->sessionKey.length() > 0)
        this->disconnectFromServer();
    delete ui;
}

void MainWindow::applicationStarted()
{
    //this->trayIcon->showMessage("", tr("Клиент техподдержки Сфера запущен"), QSystemTrayIcon::Information, 10 * 1000);
}

void MainWindow::changeEvent ( QEvent * event )
{
    if (event->type() == QEvent::WindowStateChange)
    {
            if (windowState() == Qt::WindowMinimized)
            {
                QCloseEvent *e = new QCloseEvent();
                e->setAccepted(false);
                qApp->postEvent(this, e);
            }
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (trayIcon->isVisible() && !event->isAccepted()) {
        hide();
        event->ignore();
    }
    else
    {
        int res = QMessageBox::question(0, tr("Attention!"),tr("Exit from application?"), tr("Yes"), tr("No"));
        switch(res)
        {
        case 0 :{
                if(this->connected)
                {
                    this->disconnectFromServer();
                }
                qApp->quit();
                break;
            }
        case 1 :{
                event->ignore();
                break;
            }
        }
    }

}

void MainWindow::setupMainToolBar()
{
    ui->mainToolBar->addAction(tr("Connect"), this, SLOT(connectToServer()));
    ui->mainToolBar->addAction(tr("Disconnect"), this, SLOT(disconnectFromServer()));
    ui->mainToolBar->actions().at(1)->setEnabled(false);
    ui->mainToolBar->addSeparator();
    ui->mainToolBar->addAction(tr("New ticket"), this, SLOT(newTicketDialog()));
    ui->mainToolBar->actions().at(3)->setEnabled(false);
}

LoginDialog * MainWindow::connectToServer()
{
    if(!this->connected)
    {
        if(!this->isVisible())
            this->showNormal();
        LoginDialog *loginDialog = new LoginDialog(this);
        QObject::connect(loginDialog, SIGNAL(loginAnonymous(const QString &)), this,  SLOT(doLoginAnonymous(const QString &)));
        QObject::connect(loginDialog, SIGNAL(loginRegistered(const QString &, const QString &)), this,  SLOT(doLoginRegistered(const QString &, const QString &)));
        loginDialog->show();
        loginDialog->raise();
        return loginDialog;
    }

    return NULL;
}

void MainWindow::disconnectFromServer()
{
    if(this->connected)
    {
        if(this->timer && !this->isHidden())
        {
            this->timer->stop();
        }

        QList<QVariant> empty;
        this->model->setupModel(empty);

        this->xmlRpc_userLogout(this->sessionKey);
        this->sessionKey.clear();
        afterStarted = false;
        ui->textBrowser->clear();
   }
}

void MainWindow::doLoginAnonymous(const QString & email)
{
    if(!this->connected)
    {
        this->xmlRpc_userAuth(email);
    }
}

void MainWindow::doLoginRegistered(const QString & login, const QString & password)
{
    if(!this->connected)
    {
        this->loginName = login;
        //QCryptographicHash *hash =  new QCryptographicHash(QCryptographicHash::Sha1);
        //hash->addData(password.toAscii(), password.length());
        //QByteArray sha1 = hash->result();
        //this->xmlRpc_userAuth(login, sha1.toHex());
        this->xmlRpc_userAuth(login, password);
    }
}

void MainWindow::doLoginComplete()
{
    ui->mainToolBar->actions().at(0)->setEnabled(false);
    ui->mainToolBar->actions().at(1)->setEnabled(true);
    ui->mainToolBar->actions().at(3)->setEnabled(true);
    this->connectAction->setEnabled(false);
    this->disconnectAction->setEnabled(true);

    this->timer->setInterval(1*1*60*1000);
    connect(this->timer, SIGNAL(timeout()), this, SLOT(doGetUserRequestsList()));
    this->xmlRpc_GetUserRequestsList();
    connected = true;
    //trayIcon->showMessage(tr("Клиент техподдержки Sphere"), tr("Соединение с сервером установлено"), QSystemTrayIcon::Information,
    //                      10 * 1000);

    PlasmaNotify *notify = new PlasmaNotify("sphere-client", tr("Helpdesk agent Sphere"), tr("Connect with server estabilished"), \
                                            ":/images/icons/helpdesk_64.png", 5 * 1000);
    notify->send();
}

void MainWindow::doLogoutComplete()
{
    ui->mainToolBar->actions().at(0)->setEnabled(true);
    ui->mainToolBar->actions().at(1)->setEnabled(false);
    ui->mainToolBar->actions().at(3)->setEnabled(false);
    this->connectAction->setEnabled(true);
    this->disconnectAction->setEnabled(false);
    connected = false;
}

void MainWindow::doGetUserRequestsList()
{
    this->xmlRpc_GetUserRequestsList();
}

void MainWindow::xmlRpc_userAuth(QString email)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant userEmail = email;
    client->request("xmlrpc.userAuth2", 0, userEmail);
    this->anonymous = true;
}

void MainWindow::xmlRpc_userAuth(const QString & login, const QString & password)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();
    client->request("xmlrpc.userAuth2", 1, login, password);
    this->anonymous = false;
}

void MainWindow::xmlRpc_userLogout(const QString & key)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = key;
    client->request("xmlrpc.userLogout2", sessionKey);
}

void MainWindow::xmlRpc_GetUserRequestsList()
{
    this->timer->stop();
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    client->request("xmlrpc.getUserRequestsList", sessionKey);
}

void MainWindow::xmlRpc_UpdateTicketTrack(int ticketId, int updateType, QVariant updateData)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    Variant id = ticketId;
    Variant type = updateType;
    Variant data = updateData.toString();
    client->request("xmlrpc.updateTicketTrack", sessionKey, id, updateType, data);
}


void MainWindow::xmlRpc_CreateNewTicket(const QString & subject, const QString & body, const QString & filePath)
{
    Variant fileName = "";
    Variant fileData = "";

    if(filePath.length() > 0)
    {

        QFile file(filePath);
        if (!file.open(QIODevice::ReadOnly))
        {
            QMessageBox::warning(0,
                                     tr("Error!"),
                                     tr("Cannot read a file ")+filePath,
                                     QMessageBox::Ok,Qt::NoButton);
            return;
        }

        QByteArray data;
        while (!file.atEnd())
        {
            data = file.readAll();
        }

        QFileInfo fileInfo(filePath);
        fileName = fileInfo.completeBaseName()+tr(".")+fileInfo.completeSuffix();
        fileData = data.toBase64();
    }

    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    Variant ticketSubject = subject;
    Variant ticketBody = body;
    Variant ticketSoftwareInfo = (this->settingsObj->userSoftwareInfo);

    QList<Variant> params;
    params.append(sessionKey);
    params.append(ticketSubject);
    params.append(ticketBody);
    params.append(ticketSoftwareInfo);
    params.append(fileName);
    params.append(fileData);
    client->request(params,"xmlrpc.newTicket");
}

void MainWindow::xmlRpc_GetTicketFollowups(int ticketId)
{
    xmlrpc::Client * client = this->prepareXmlRpcClient();

    Variant sessionKey = this->sessionKey;
    Variant id = ticketId;
    client->request("xmlrpc.getTicketFollowups", sessionKey, id);
}

xmlrpc::Client *MainWindow::prepareXmlRpcClient()
{
    xmlrpc::Client *client = new Client(this);

    QObject::connect(client,SIGNAL(done(int, QVariant)),
                            this, SLOT(onDone(int, QVariant)));
    QObject::connect(client,SIGNAL(failed( int, int, QString )),
                 this, SLOT(processFault( int, int, QString )) );

    QUrl url(this->settingsObj->helpdeskServerUrl);
    client->setHost(url.host(), 80, url.path());
    client->setUserAgent("sphere-client");

    return client;
}

void MainWindow::processFault( int requestId, int errorCode, QString errorString )
{
    Q_UNUSED( requestId );

    QMessageBox::warning(this, tr("Request error"),
            QString(QObject::tr("XML-RPC request error.\n\nError code: %1\n%2\n")) \
            .arg(errorCode).arg(errorString),
            QMessageBox::Ok );

    QCoreApplication::exit(-1);
}

void MainWindow::onDone(int, QVariant variant)
{
    // QVariant - карта со значениями
    QMultiMap<QString , QVariant> map(variant.toMap());
    QString flerror = map.value("flerror").toString(); // код ошибки
    // сообщение (объяснение ошибки, если она была)
    QString message = map.value("message").toString();

    if(flerror.size() != 0)
    {
        QMessageBox::warning(0,
                             flerror,
                             message,
                             QMessageBox::Ok,Qt::NoButton);

        QCoreApplication::exit(-1);
        return;
    }

    QString method = map.value("xmlrpc_response.method").toString();

    if(method == QString("userAuth"))
    {
        if(map.value("xmlrpc_response.loginResult").toInt() == 1)
        {
            this->sessionKey = map.value("xmlrpc_response.key").toString();
            this->doLoginComplete();
        }
        else
        {
            this->anonymous = false;
            QMessageBox::warning(0,
                                 tr("Login failure!"),
                                 tr("Unknown user name or password! Please, enter valid user name and password and make login procedure again."),
                                 QMessageBox::Ok,Qt::NoButton);
            LoginDialog *loginDialog = this->connectToServer();
            loginDialog->setType(1);
            loginDialog->setLoginName(this->loginName);
        }

        return;
    }

    if(method == QString("userLogout"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {
            this->doLogoutComplete();
        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Error!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }
        return;
    }

    if(method == QString("newTicket"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {
            QMessageBox::information(this, tr("Information"),
                                     tr("Your ticket for helpdesk service has been registered by id ")+map.value("xmlrpc_response.ticketId").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            this->xmlRpc_GetUserRequestsList();
        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Error!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }
        return;
    }

    if(method == QString("getUserRequestsList"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {
            QList<QVariant> rows = map.value("xmlrpc_response.rows").toList();
            this->model->setupModel(rows);

            if(rows.count() > 0)
            {
                if(!ui->tableView->currentIndex().isValid())
                {
                    QModelIndex idx = this->model->index(0,0);
                    ui->tableView->setCurrentIndex(idx);
                    this->on_tableView_clicked(idx);
                }
            }

            ui->tableView->horizontalHeader()->setResizeMode(ui->tableView->horizontalHeader()->logicalIndexAt(0), QHeaderView::ResizeToContents);
            ui->tableView->horizontalHeader()->setResizeMode(1, QHeaderView::Stretch);
            ui->tableView->horizontalHeader()->setResizeMode(2, QHeaderView::ResizeToContents);
            ui->tableView->horizontalHeader()->setResizeMode(3, QHeaderView::ResizeToContents);
            ui->tableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);

            //счетчики
            bool flag = counterNew != map.value("xmlrpc_response.counterNew").toInt() ||
                        counterAssign != map.value("xmlrpc_response.counterAssign").toInt() ||
                        counterPlan != map.value("xmlrpc_response.counterPlan").toInt() ||
                        counterWaiting != map.value("xmlrpc_response.counterWaiting").toInt() ||
                        counterSolved != map.value("xmlrpc_response.counterSolved").toInt() ||
                        counterClosed != map.value("xmlrpc_response.counterClosed").toInt() ||
                        counterFollowUps != map.value("xmlrpc_response.counterFollowUps").toInt();
            if(flag)
            {
                QString str = "";

                afterStarted = true;
                if(afterStarted)
                {
                    //if(counterNew < map.value("xmlrpc_response.counterNew").toInt())
                    //    str = str + tr("новых: %1").arg(QString::number(map.value("xmlrpc_response.counterNew").toInt() - counterNew));

                    if(counterAssign < map.value("xmlrpc_response.counterAssign").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("assigned: %1").arg(QString::number(map.value("xmlrpc_response.counterAssign").toInt() - counterAssign));
                    }
                    if(counterPlan < map.value("xmlrpc_response.counterPlan").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("planned: %1").arg(QString::number(map.value("xmlrpc_response.counterPlan").toInt() - counterPlan));
                    }
                    if(counterWaiting < map.value("xmlrpc_response.counterWaiting").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("waiting %1").arg(QString::number(map.value("xmlrpc_response.counterWaiting").toInt() - counterWaiting));
                    }
                    if(counterSolved < map.value("xmlrpc_response.counterSolved").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("solved: %1").arg(QString::number(map.value("xmlrpc_response.counterSolved").toInt() - counterSolved));
                    }
                    if(counterClosed < map.value("xmlrpc_response.counterClosed").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("closed: %1").arg(QString::number(map.value("xmlrpc_response.counterClosed").toInt() - counterClosed));
                    }
                    if(counterFollowUps < map.value("xmlrpc_response.counterFollowUps").toInt())
                    {
                        if(str.length() > 0)
                            str = str + tr("\n");
                        str = str + tr("new followups: %1").arg(QString::number(map.value("xmlrpc_response.counterFollowUps").toInt() - counterClosed));
                    }

                    if(str.length() > 0)
                    {
                        PlasmaNotify *notify = new PlasmaNotify("sphere-client", tr("Helpdesk agent Sphere"), tr("Information about tickets has been updated: %1").arg("\n\n"+str), ":/images/icons/helpdesk_64.png", 10000);
                        notify->send();
                    }
                }

                counterNew = map.value("xmlrpc_response.counterNew").toInt();
                counterAssign = map.value("xmlrpc_response.counterAssign").toInt();
                counterPlan = map.value("xmlrpc_response.counterPlan").toInt();
                counterWaiting = map.value("xmlrpc_response.counterWaiting").toInt();
                counterSolved = map.value("xmlrpc_response.counterSolved").toInt();
                counterClosed = map.value("xmlrpc_response.counterClosed").toInt();
                counterFollowUps = map.value("xmlrpc_response.counterFollowUps").toInt();

                afterStarted = true;
            }

            this->timer->start();
        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Error!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }

        return;
    }

    if(method == QString("updateTicketTrack"))
    {
        if(map.value("xmlrpc_response.result").toInt() == 1)
        {

        }
        else
        {
            QMessageBox::warning(0,
                                     tr("Error!"),
                                     map.value("xmlrpc_response.resultMessage").toString(),
                                     QMessageBox::Ok,Qt::NoButton);
            QCoreApplication::exit(-1);
        }
        return;
    }

    QMessageBox::warning(0,
                             tr("Error!"),
                             tr("Unknown answer from XML-RPC server!"),
                             QMessageBox::Ok,Qt::NoButton);
    QCoreApplication::exit(-1);
}

void MainWindow::on_tableView_clicked(QModelIndex index)
{
    QString message;
    if(index.isValid())
    {
        QTextBrowser txt;
        txt.setHtml(this->model->data(this->model->index(index.row(), 5), Qt::DisplayRole).toString());
        txt.setHtml(txt.toPlainText());
        QString msg = this->model->data(this->model->index(index.row(), 4), Qt::DisplayRole).toString();

        message = tr("<div>&nbsp;</div><div style=\" ;text-indent:-5px;\"><span style=\" font-size:8pt; font-weight:600; color:#ffffff; background-color:#f47e65;position:relative;left:-10px;\">&nbsp;&nbsp;Request&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span></div><div style=\" margin-left:5px; font-size:8pt;\">%1</div><br><div style=\"text-indent:-5px;\"><span style=\" font-size:8pt; font-weight:600; color:#ffffff; background-color:#69ad71;\">&nbsp;&nbsp;Solution&nbsp;&nbsp;&nbsp;&nbsp;</span></div><div style=\" margin-left:5px; font-size:8pt;\">%2</div><br>")
                  .arg(this->model->data(this->model->index(index.row(), 4), Qt::DisplayRole).toString())
                  .arg(txt.toPlainText());

        if(this->model->data(this->model->index(index.row(), 3), Qt::UserRole) != this->model->data(this->model->index(index.row(), 8), Qt::UserRole))
        {
            this->model->setData(this->model->index(index.row(), 8), this->model->data(this->model->index(index.row(), 3), Qt::UserRole), Qt::UserRole);
            this->xmlRpc_UpdateTicketTrack(this->model->data(this->model->index(index.row(), 0), Qt::DisplayRole).toInt(), 0, this->model->data(this->model->index(index.row(), 3), Qt::UserRole));
        }
    }
    else
        message = "";

    ui->textBrowser->setHtml(message);
}

void MainWindow::doRefreshTicketBody()
{
    this->on_tableView_clicked(ui->tableView->currentIndex());
}

void MainWindow::createActions()
{
    minimizeAction = new QAction(tr("Mi&nimize"), this);
    connect(minimizeAction, SIGNAL(triggered()), this, SLOT(hide()));

    maximizeAction = new QAction(tr("Ma&ximize"), this);
    connect(maximizeAction, SIGNAL(triggered()), this, SLOT(showMaximized()));

    restoreAction = new QAction(tr("Show"), this);
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

    quitAction = new QAction(tr("&Exit"), this);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(doQuit()));

    connectAction = new QAction(tr("&Connect"), this);
    connect(connectAction, SIGNAL(triggered()), this, SLOT(connectToServer()));

    disconnectAction = new QAction(tr("Disconnect"), this);
    connect(disconnectAction, SIGNAL(triggered()), this, SLOT(disconnectFromServer()));
    this->disconnectAction->setEnabled(false);
}


void MainWindow::createTrayIcon()
{
    trayIconMenu = new QMenu(this);
    //trayIconMenu->addAction(minimizeAction);
    //trayIconMenu->addAction(maximizeAction);
    trayIconMenu->addAction(connectAction);
    trayIconMenu->addAction(disconnectAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(restoreAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(quitAction);

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setIcon(QIcon(":/images/icons/helpdesk_64.png"));
    trayIcon->setContextMenu(trayIconMenu);

    setWindowIcon(trayIcon->icon());
    trayIcon->setToolTip(tr("Helpdesk agent Sphere"));


    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
                this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
    connect(trayIcon, SIGNAL(messageClicked()), this, SLOT(messageClicked()));
}

void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
    case QSystemTrayIcon::Trigger:
        break;
    case QSystemTrayIcon::DoubleClick:
        this->showNormal();
        this->activateWindow();
        break;
    case QSystemTrayIcon::MiddleClick:
        break;
    default:
        ;
    }
}

void MainWindow::messageClicked()
{
    this->iconActivated(QSystemTrayIcon::DoubleClick);
}

NewTicketDialog *MainWindow::newTicketDialog()
{
    NewTicketDialog *dialog = new NewTicketDialog(this);
    QObject::connect(dialog, SIGNAL(saveTicket(const QString &, const QString &, const QString &)), this,  SLOT(doSaveTicket(const QString &, const QString &, const QString &)));
    dialog->show();
    dialog->raise();
    return dialog;
}

void MainWindow::doSaveTicket(const QString & ticketSubj, const QString & ticketBody, const QString & filePath)
{
    this->xmlRpc_CreateNewTicket(ticketSubj, ticketBody, filePath);
}

void MainWindow::moveWindowToCenter() {
    QRect frect = frameGeometry();
    frect.moveCenter(QDesktopWidget().availableGeometry().center());
    move(frect.topLeft());
}

void MainWindow::doQuit()
{
    this->showNormal();
    QCloseEvent *e = new QCloseEvent();
    qApp->postEvent(this, e);
}

void MainWindow::on_tableView_doubleClicked(QModelIndex index)
{
    if(!this->anonymous)
    {
        FollowupsListModel *followupsModel = new FollowupsListModel(this->model->data(this->model->index(index.row(), 0), Qt::UserRole).toInt(), this->loginName);

        ChatDialog *chatDialog = new ChatDialog(followupsModel, this->sessionKey, this->settingsObj, this);

        QObject::connect(chatDialog, SIGNAL(updateFollowupsCount()), this,  SLOT(updateFollowupsCount()));

        chatDialog->show();
        chatDialog->raise();
        if(this->model->data(this->model->index(index.row(), 9), Qt::UserRole).toInt() < this->model->data(this->model->index(index.row(), 10), Qt::UserRole).toInt())
        {
            this->model->setData(this->model->index(index.row(), 9), this->model->data(this->model->index(index.row(), 10), Qt::UserRole), Qt::UserRole);
            this->xmlRpc_UpdateTicketTrack(this->model->data(this->model->index(index.row(), 0), Qt::DisplayRole).toInt(), 1, this->model->data(this->model->index(index.row(), 10), Qt::UserRole));
            this->counterFollowUps = 0;
        }
    }
    else
    {
        QMessageBox::warning(this,
                                 tr("Information"),
                                 tr("Cannot view tickets followups for anonymous users. Please, register on http://suppot.rosalab.ru and enter you request again."),
                                 QMessageBox::Ok,Qt::NoButton);
    }

}

void MainWindow::updateFollowupsCount()
{
    this->model->setData(this->model->index(ui->tableView->currentIndex().row(), 9), this->model->data(this->model->index(ui->tableView->currentIndex().row(), 9), Qt::UserRole).toInt()+1, Qt::UserRole);
    this->model->setData(this->model->index(ui->tableView->currentIndex().row(), 10), this->model->data(this->model->index(ui->tableView->currentIndex().row(), 10), Qt::UserRole).toInt()+1, Qt::UserRole);
    this->xmlRpc_UpdateTicketTrack(this->model->data(this->model->index(ui->tableView->currentIndex().row(), 0), Qt::DisplayRole).toInt(), 2, 0);
}

void MainWindow::paintEvent (QPaintEvent *pEvent)
{
//    if (this->wImageLabel && !pEvent->region().isEmpty())
//    {
//        QPainter painter (this);
//        QRect wRect (ui->textBrowser->frameRect().topLeft(), ui->textBrowser->frameRect().bottomRight());
//        painter.drawImage (wRect, *this->wImageLabel);
//    }
}
