#include "requestslistmodel.h"
#include <QIcon>
#include <QFont>
#include <QDateTime>

RequestsListModel::RequestsListModel(QObject *parent) :
    QAbstractTableModel(parent)
{
}

int RequestsListModel::rowCount(const QModelIndex &parent) const
{
    if(parent.isValid())
        return 0;
    return this->rows.count();
}

int RequestsListModel::columnCount(const QModelIndex &parent) const
{
    if(parent.isValid())
        return 0;
    if(this->rows.count() > 0)
    {
        QMultiMap<QString , QVariant> row = this->rows.at(0).toMap();
        return row.count();
    }
    else
        return 11;
}

QVariant RequestsListModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
            return QVariant();

    if(role == Qt::TextAlignmentRole)
    {
        int value;
        if(index.column() == 0)
            value = int(Qt::AlignTop | Qt::AlignRight);
        else
            value = int(Qt::AlignTop | Qt::TextWordWrap);

        return value;
    }

    if (role == Qt::FontRole)
    {
        QVariant value;

        if( data(this->index(index.row(), 3), Qt::UserRole) != data(this->index(index.row(), 8), Qt::UserRole))
        {
            QFont font;
            font.setBold(true);
            value = qVariantFromValue(font);
        }

        return value;
    }

    if (role == Qt::DecorationRole && index.column() == 3)
    {
        QMultiMap<QString , QVariant> row = this->rows.at(index.row()).toMap();
        QVariant value;

        if( row.value("ticketFollowUps").toInt() > row.value("trackTicketFollowUps").toInt() )
        {
            value = QIcon(":/images/icons/new_mess.png");
        }
        return value;
    }

    if (role == Qt::DisplayRole || role == Qt::UserRole)
    {
        QMultiMap<QString , QVariant> row = this->rows.at(index.row()).toMap();
        QVariant value;

        switch(index.column())
        {
        //ID
        case 0 : {
                value = row.value("ticket_id");
                break;
            }
            //Тема
        case 1 : {
                value = row.value("subject");
                break;
            }
            //Дата создания
        case 2 : {
                if(role == Qt::UserRole)
                {
                    value = row.value("ticketAddedDate");
                }
                else
                {
                    QVariant dateTime = row.value("ticketAddedDate");
                    value = dateTime.toDateTime().toString("dd.MM.yyyy hh:mm:ss");
                    if(QDate::currentDate() == dateTime.toDate())
                            value = tr("today");
                    if(QDate::currentDate().addDays(-1) == dateTime.toDate())
                            value = tr("yesterday");
                }
                break;
            }
            //Статус
        case 3 : {
                if (role == Qt::DisplayRole)
                {
                    if(row.value("status") == QVariant("new"))
                        value = QVariant(tr("new"));
                    if(row.value("status") == QVariant("assign"))
                        value = QVariant(tr("assigned"));
                    if(row.value("status") == QVariant("plan"))
                        value = QVariant(tr("planned"));
                    if(row.value("status") == QVariant("waiting"))
                        value = QVariant(tr("waiting"));
                    if(row.value("status") == QVariant("solved"))
                        value = QVariant(tr("solved"));
                    if(row.value("status") == QVariant("closed"))
                        value = QVariant(tr("closed"));
                }

                if (role == Qt::UserRole)
                {
                    value = row.value("status");
                }

                break;
            }

            //Тело заявки
        case 4 : {
                value = row.value("content").toString();
                break;
            }
            //Решение
        case 5 : {
                value = row.value("solution");
                break;
            }
            //Дата изменения
        case 6 : {
                if(role == Qt::UserRole)
                {
                    value = row.value("ticketModifiedDate");
                }
                else
                {
                    QVariant dateTime = row.value("ticketModifiedDate");
                    value = dateTime.toDateTime().toString("dd.MM.yyyy hh:mm:ss");
                }
                break;
            }
            //Дата предыдущего изменения
        case 7 : {
                if(role == Qt::UserRole)
                {
                    value = row.value("trackTicketModifiedDate");
                }
                else
                {
                    QVariant dateTime = row.value("trackTicketModifiedDate");
                    value = dateTime.toDateTime().toString("dd.MM.yyyy hh:mm:ss");
                }
                break;
            }
            //Предыдущий статус заявки
        case 8 : {
                value = row.value("trackTicketStatusId");
                break;
            }
            //Предыдущее кол-во комментариев
        case 9 : {
                value = row.value("trackTicketFollowUps");
                break;
            }
            //Kол-во комментариев
        case 10 : {
                value = row.value("ticketFollowUps");
                break;
            }
        }

        return value;
    }

    return QVariant();

    //return QAbstractTableModel::data(index, role);
}

void RequestsListModel::setupModel(const QList<QVariant> & rows)
{
    this->rows = rows;
    //emit dataChanged(this->index(0,0), this->index(this->rowCount()-1, this->columnCount()-1));
    //this->reset();
    emit layoutChanged();
}

QVariant RequestsListModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole)
    {
        if(orientation == Qt::Horizontal)
        {
            switch(section)
            {
            case 0 : {
                    return tr("№");
                    break;
                }
            case 1 : {
                    return tr("Subject");
                    break;
                }
            case 2 : {
                    return tr("Date of creation");
                    break;
                }
            case 3 : {
                    return tr("Status");
                    break;
                }
            case 4 : {
                    return tr("Text");
                    break;
                }
            case 5 : {
                    return tr("Solution");
                    break;
                }
            case 6 : {
                    return tr("Date of modification");
                    break;
                }
                //Дата предыдущего изменения
            case 7 : {
                    return tr("Previous modification date");
                    break;
                }
                //Предыдущий статус заявки
            case 8 : {
                    return tr("Previous status");
                    break;
                }
                //Предыдущее кол-во комментариев
            case 9 : {
                    return tr("Previous followups");
                    break;
                }
                //Kол-во комментариев
            case 10 : {
                    return tr("Followups");
                    break;
                }
            }
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

bool RequestsListModel::setData ( const QModelIndex & index, const QVariant & value, int role)
{
    if (role == Qt::UserRole)
    {
        if(index.column() == 8)
        {
            QMultiMap<QString , QVariant> map = this->rows.at(index.row()).toMap();
            map.replace("trackTicketStatusId", value);
            this->rows.replace(index.row(), map);
            emit dataChanged(this->index(index.row(),0), this->index(index.row(), this->columnCount()-1));
            return true;
        }

        if(index.column() == 9)
        {
            QMultiMap<QString , QVariant> map = this->rows.at(index.row()).toMap();
            map.replace("trackTicketFollowUps", value);
            this->rows.replace(index.row(), map);
            emit dataChanged(this->index(index.row(),0), this->index(index.row(), this->columnCount()-1));
            return true;
        }

        if(index.column() == 10)
        {
            QMultiMap<QString , QVariant> map = this->rows.at(index.row()).toMap();
            map.replace("ticketFollowUps", value);
            this->rows.replace(index.row(), map);
            emit dataChanged(this->index(index.row(),0), this->index(index.row(), this->columnCount()-1));
            return true;
        }
    }

    return false;
}

